<?php declare(strict_types=1);

namespace Infomaker\Everyware\Twig\Extension;

use Infomaker\Everyware\Support\Str;
use Twig\TwigFilter;

/**
 * Class Filters
 * @package Infomaker\Everyware\Twig\Extension
 */
class Filters implements ExtensionResourceInterface
{
    /** @var array */
    private $filters = [];

    public function __construct()
    {
        $this->addResource('class_string', [$this, 'classString']);
        $this->addResource('trim_array', [$this, 'trimArray']);
        $this->addResource('spacey', [$this, 'spacey']);
    }

    public function classString($list)
    {
        $list = $this->getValidArray($list);

        return trim(preg_replace('!\s+!', ' ', implode(' ', $list)));
    }

    public function trimArray($array)
    {
        $array = array_map([$this, 'getValidArray'], $this->getValidArray($array));
        $array = array_merge(...$array);

        return array_filter($array, static function ($item) {
            return Str::notEmpty((string)$item);
        });
    }

    /**
     * Protect whitespace between tags, so the `spaceless` filter won't remove it.
     *
     * Any sequence of whitespace characters will be transformed into a single, normal (but HTML encoded) space.
     *
     * @param string|null $html
     *
     * @return string
     */
    public function spacey(string $html = null): string
    {
        if ($html === null) {
            return '';
        }

        return preg_replace('|>\s+<|', '>&#x0020;<', $html);
    }

    private function getValidArray($value)
    {
        if ( ! \is_iterable($value)) {
            $value = (array)$value;
        }

        if ($value instanceof \Traversable) {
            return iterator_to_array($value);
        }

        return $value;
    }

    /**
     * @inheritDoc
     */
    public function addResource(string $name, $callable = null, array $options = []): void
    {
        $this->filters[$name] = new TwigFilter($name, $callable, $options);
    }

    /**
     * @inheritDoc
     */
    public function getResourceList(): array
    {
        return $this->filters;
    }
}
